/*****************************************************************************/
/*
                                  cache.h
*/
/*****************************************************************************/

#ifndef CACHE_H_LOADED
#define CACHE_H_LOADED 1

#include "wasd.h"
#include "file.h"

/**************/
/* structures */
/**************/

struct CacheStruct
{
   struct ListEntryStruct  CacheListEntry;

   struct CacheStruct  *HashCollisionNextPtr,
                       *HashCollisionPrevPtr;

   boolean  /* cache is currently being loaded for this path/file */
            CacheLoading,
            /* data load completed OK */
            CacheLoadValid,
            /* cache entry has been invalidated to scavenge memory */
            CacheScavenged,
            /* data in cache is valid and can be use for requests */
            CacheValid,
            /* remove allocated data buffer as soon as not in use */
            Purge,
            /* also remove cache entry */
            PurgeCompletely;

   unsigned short /* first free byte in end-of-file VBN */
                  FirstFreeByte;

   unsigned long  /* size of the block pointed to by 'ContentPtr' */
                  CacheSize,
                  /* number of times current allocated buffer been reused */
                  ContentBufferUsedCount,
                  /* number of times buffer has been reallocated */
                  ContentBufferReallocatedCount,
                  /* length of data (file contents) pointed to 'ContentPtr' */
                  ContentLength,
                  /* length of "dev:" in file name */
                  DeviceNameLength,
                  /* virtual block number of end-of-file */
                  EndOfFileVbn,
                  /* length of "dev:[dir]name.type;" */
                  FileNameLength,
                  /* hash value of characters in path */
                  HashValue,
                  /* number of times cache used */
                  HitCount,
                  /* number of times cache hit but file unmodified */
                  HitNotModifiedCount,
                  /* cache entry is in use and should not be modified */
                  InUseCount,
                  /* length of "name.type" in file name */
                  NameTypeLength,
                  /* length of path */
                  PathLength,
                  /* number of times the entry has been validated */
                  ValidatedCount;

   unsigned long  /* last hit quadword */
                  HitBinaryTime [2],
                  /* when cache data loaded quadword */
                  LoadBinaryTime [2],
                  /* last modified quadword */
                  ModifiedBinaryTime [2],
                  /* last validated quadword */
                  ValidateBinaryTime [2];

   unsigned char  /* 6 byte directory id field from the NAM block */
                  FileNam_did [6];

   unsigned char  /* points to file contents */
                  *ContentPtr,
                  /* points to file's MIME content type */
                  *ContentTypePtr,
                  /* points to "dev:" in file name */
                  *DeviceNamePtr,
                  /* points to "[dir]name,type" in file name */
                  *NameTypePtr;

   char  /* path representing the cached file */
         Path [128],
         /* RMS file name */
         FileName [128];

   /* structure for ACP info */
   struct FileAcpStruct  FileAcpData;
};

/***********************/
/* function prototypes */
/***********************/

CacheAcpInfoAst (struct RequestStruct*);
CacheBegin (struct RequestStruct*);
CacheEnd (struct RequestStruct*);
CacheInit ();
CacheLoadBegin (struct RequestStruct*, int);
CacheLoadEnd (struct RequestStruct*, struct FileTaskStruct*);
CacheSearch (struct RequestStruct*);
CacheNext (struct RequestStruct*);
CachePurge (boolean, int*, int*);
CacheReport (struct RequestStruct*, void*);
CacheZeroCounters ();

#endif /* CACHE_H_LOADED */

/*****************************************************************************/
