/*****************************************************************************/
/*
                                  DCL.h
*/     
/*****************************************************************************/

#ifndef DCL_H_LOADED
#define DCL_H_LOADED 1

#include "wasd.h"

/* no world or group access */
#define SubprocessMbxProtectionMask 0xff00

#define DCL_FIND_SCRIPT_BEGIN   1
#define DCL_FIND_SCRIPT_EXE     2
#define DCL_FIND_SCRIPT_RUNTIME 3

/**************/
/* structures */
/**************/

struct DclScriptNameCacheStruct
{
   struct  ListEntryStruct  DclListEntry;

   int  /* number of times "hit" since last cache file name load */
        HitCount,
        /* number of times cache entry used */
        UsageCount;

   unsigned long  /* last time loaded/hit */
                  LastBinaryTime [2];

   char  /* script name */
         ScriptName [SCRIPT_NAME_SIZE],
         /* when finding a script file */
         ScriptExpandedFileName [256];

   char  /* points to a string denoting how the script is to be invoked */
         *ScriptRunTimePtr;
};

struct DclTaskStruct
{
   struct  ListEntryStruct  DclListEntry;

   boolean  /* add a newline to each record from a script subprocess */
            AddNewline,
            /* record-by-record output from DCL, flush immediately */
            BufferRecords,
            /* this is a "standard" CGI script */
            CgiScript,
            /* this is a "CGI-plus" script */
            CgiPlusScript,
            /* script read variables (i.e. became active) */
            CgiScriptActivated,
            /* script provided response (i.e. sent some output) */
            CgiScriptResponded,
            /* output is text (plain or html) */
            ContentTypeText,
            /* DclFindScript() is currently using the structure */
            FindScript,
            /* this is merely a DCL command (from SSI) */
            IsDclCommand,
            /* when the task completes the subprocess is deleted */
            MarkedForDelete;
            
   int  /* copied from request structure for WATCHing purposes */
        AdminWatchItem,
        /* number of times the CGIplus script has been used */
        CgiPlusUsageCount,
        /* remaining content in POST script */
        ContentLength,
        /* length of the DCL EOF string */
        DclEofLength,
        /* */
        FindScriptRunTimeIdx,
        /* */
        FindScriptState,
        /* used by the DclSupervisor() */
        LifeTimeCount,
        /* count of I/Os CGIplus variables reads still outstanding */
        QueuedCgiPlusIn,
        /* count of HTTP$INPUT reads still outstanding */
        QueuedHttpInput,
        /* count of I/Os between subprocess and HTTPd still outstanding */
        QueuedSysCommand,
        /* when the count of I/Os is considered task finished */
        QueuedSysCommandAllowed,
        /* count of I/Os between subprocess and HTTPd still outstanding */
        QueuedSysOutput,
        /* size of DCL subprocess' SYS$OUTPUT buffer */
        SysOutputSize,
        /* total number of times the task slot was used */
        TotalUsageCount,
        /* number of subprocess has been a zombie */
        ZombieCount;

   unsigned short  /* channel to subprocess' CGIPLUSIN mailbox */
                   CgiPlusInChannel,
                   /* channel to subprocess' HTTP$INPUT mailbox */
                   HttpInputChannel,
                   /* channel to subprocess' SYS$INPUT mailbox */
                   SysCommandChannel,
                   /* channel to subprocess' SYS$OUTPUT mailbox */
                   SysOutputChannel;

   unsigned long  /* subprocess' exit status */
                  SubprocessCompletionStatus,
                  /* subprocess' PID */
                  SubprocessPid;

   unsigned long  /* last time CGIplus script was used */
                  LastUsedBinaryTime [2];

   unsigned char  /* content pointer in POST script */
                  *ContentPtr,
                  /* points to a string denoting how the script is invoked */
                  *ScriptRunTimePtr,
                  /* heap storage for buffering subprocess' SYS$OUTPUT */
                  *SysOutputPtr;

   unsigned char  /* device name of subprocess' CGIPLUSIN mailbox */
                  CgiPlusInDevName [64],
                  /* DCL command (from SSI) */
                  DclCommand [256],
                  /* DCL EOF string */
                  DclEof [64],
                  /* device name of subprocess' HTTP input mailbox */
                  HttpInputDevName [64],
                  /* script name */
                  ScriptName [SCRIPT_NAME_SIZE],
                  /* when finding a script file */
                  ScriptExpandedFileName [256],
                  /* the name of the script file being executed */
                  ScriptFileName [256],
                  /* device name of subprocess' input mailbox */
                  SysCommandDevName [64],
                  /* device name of subprocess' output mailbox */
                  SysOutputDevName [64];

   struct AnIOsb  CgiPlusInIOsb,
                  HttpInputIOsb,
                  SysCommandIOsb,
                  SysOutputIOsb;

   /* string descriptors for the various mailbox device names */
   struct dsc$descriptor_s  CgiPlusInDevNameDsc;
   struct dsc$descriptor_s  HttpInputDevNameDsc;
   struct dsc$descriptor_s  SysCommandDevNameDsc;
   struct dsc$descriptor_s  SysOutputDevNameDsc;

   /* RMS structures when finding a script file */
   struct FAB  SearchFab;
   struct NAM  SearchNam;

   /* pointer a request using this DCL task structure */
   struct RequestStruct  *RequestPtr;

   /* pointer to function, used for specifying the next task */
   void (*NextTaskFunction)(struct RequestStruct*);
};

/***********************/
/* function prototypes */
/***********************/

DclAbortSubprocess (struct DclTaskStruct*);

/* NOTE ... pointer to request structure! */
DclAllocateTask (struct RequestStruct*); 

/* NOTE ... pointer to request structure! */
DclBegin (struct RequestStruct*, void*, char*, char*); 

DclBeginScript (struct DclTaskStruct*);
DclCgiPlusInAst (struct DclTaskStruct*);
DclCgiPlusLifeTimeAst (struct DclTaskStruct*);
DclCgiScriptSysCommand (struct DclTaskStruct*);
DclConcludeTask (struct DclTaskStruct*, boolean);
DclDeallocateTask (struct DclTaskStruct*);
DclFindScript (struct DclTaskStruct*);
DclFindScriptEnd (struct DclTaskStruct*, boolean);
DclFindScriptParseAst (struct FAB*);
DclFindScriptSearchAst (struct FAB*);
DclHttpInputAst (struct DclTaskStruct*);
DclQioSysCommand (struct DclTaskStruct*, char*, int);
DclQioCgiPlusIn (struct DclTaskStruct*, char*, int);
DclQioSysOutput (struct DclTaskStruct*);

/* NOTE ... pointer to request structure! */
DclSysOutputToClientAst (struct RequestStruct*);

DclPlusScriptSysCommand (struct DclTaskStruct*);
DclPlusScriptSysCgiPlusIn (struct DclTaskStruct*);
DclPurgeScriptNameCache ();
DclPurgeAllCgiPlusScripts (struct RequestStruct*, void*, boolean);
DclSysCommandAst (struct DclTaskStruct*);
DclSearchScriptNameCache (struct DclTaskStruct*, struct RequestStruct*);
DclSubprocessCompletionAST (struct DclTaskStruct*);
DclSysOutputHeader (struct DclTaskStruct*);
DclSysOutputLocation (struct DclTaskStruct*);
DclSysOutputAst (struct DclTaskStruct*);
DclSysOutputWaitAst (struct DclTaskStruct*);
DclSetCgiPlusLifeTime (struct DclTaskStruct*);
DclSetZombieLifeTime (struct DclTaskStruct*);
DclUpdateScriptNameCache (struct DclTaskStruct*, struct RequestStruct*);
DclZombieLifeTimeAst (struct DclTaskStruct*);

char* DclControlPurgeAllSubprocesses (boolean);

#endif /* DCL_H_LOADED */

/*****************************************************************************/

