/*****************************************************************************/
/*
                                  Dir.h
*/
/*****************************************************************************/

#ifndef DIR_H_LOADED
#define DIR_H_LOADED 1

#include "wasd.h"
#include "file.h"

/**************/
/* structures */
/**************/

struct DirTaskStruct
{
   struct ListEntryStruct  DirTaskList;

   boolean  /* query string directive */
            AsIfNopFound,
            /* query string directive */
            AsIfNosFound,
            /* SYSUAF authenticated user has been checked to have access */
            AuthVmsUserHasAccess,
            /* disables auto-scripting appending version */
            AutoScriptEnabled,
            /* present a directory listing ("Index of") in a VMS-style */
            FormatLikeVms,
            /* */
            DirFormatBlockTotals,
            /* the original specification's file/type/version was not empty */
            DirSpecIncludedFilePart,
            /* query string directive */
            IncludeAnyReadme,
            /* */
            ListParentDir,
            /* */
            ListSubDir,
            /* make the file description a link */
            MakeDescriptionLink,
            /* HTTP header generated by directory lisitng, i.e. not embedded */
            ResponseHeaderSent,
            /* RMS parse is in use, free before finishing */
            ParseInUse;

   int  /* header, footer, both, none */
        Delimit,
        /* */
        DirAccessOkState,
        /* */
        DirAccessOkStatus,
        /* count of directories in listing */
        DirectoryCount,
        /* */
        LayoutFaoLength, 
        /* */
        LayoutHeadingLength, 
        /* */
        DirSpecLength,
        /* */
        FieldWidthCdt,
        /* */
        FieldWidthDescription,
        /* */
        FieldWidthName,
        /* */
        FieldWidthOwner,
        /* */
        FieldWidthProtection,
        /* */
        FieldWidthRdt,
        /* */
        FieldWidthSize,
        /* count of number of files in listing */
        FileCount,
        /* */
        ReadMeFileIndex,
        /* */
        SizeKilo,
        /* buffer for most recent task status */
        TaskStatusBuffer;

   unsigned long  /* for directory module, file's allocated blocks */
                  TotalAllocatedBlocks,
                  /* for directory module, file's used blocks */
                  TotalUsedBlocks;

   unsigned char  /* pointer to heap storage URL equivalent of directory */
                  *DirectoryPathPtr,
                  /* */
                  *DirFormatParentDirPath,
                  /* pointer to column headings string */
                  *LayoutHeadingPtr,
                  /* pointer to directory layout string */
                  *LayoutPtr,
                  /* pointer to sys$fao() directive string */
                  *LayoutFaoPtr,
                  /* pointer to message string describing subdirectory */
                  *MsgSubDirPtr,
                  /* pointer to request specified content-type */
                  *QueryContentTypePtr;
                  
   char  /* */
         Description [256],
         /* directory part of file specification */
         DirectoryPart [256],
         /* directory specification */
         DirPath [256],
         /* directory path specification */
         DirSpec [256],
         /* */
         ExpandedFileName [256],
         /* file part of file specification */
         FilePart [128],
         /* query string for directory module */
         QueryString [128],
         /* only when specified as "/dir1/dir2" this contains "./dir2/" */
         RelativePath [128],
         /* */
         ResultantFileName [256],
         /* prepended script name */
         ScriptName [SCRIPT_NAME_SIZE];

   /* */
   struct ContentTypeStruct  ContentType;

   /* used for retrieving file size, dates, etc. */
   struct FileAcpStruct  FileAcpData;

   /* RMS structures */
   struct FAB  FileFab;
   struct RAB  FileRab;
   struct NAM  FileNam;
   struct FAB  SearchFab;
   struct NAM  SearchNam;

   /* pointers to functions, used for specifying the next/AST task */
   void (*NextTaskFunction)(struct RequestStruct*);
   void (*DirAccessOkAstFunctionPtr)(struct RequestStruct*);
   void (*DirFormatAstFunctionPtr)(struct RequestStruct*);
};

/***********************/
/* function prototypes */
/***********************/

DirAccessOk (struct RequestStruct*);
DirBegin (struct RequestStruct*, void*, char*, char*, char*);
DirBeginDirectories (struct RequestStruct*);
DirBeginFiles (struct RequestStruct*);
DirBeginFilesParseAst (struct FAB*);
DirDirectories (struct FAB*);
DirBeginDirectoriesParseAst (struct FAB*);
DirDirectoriesAccessOkAst (struct RequestStruct*);
DirEnd (struct RequestStruct*);
DirEndDescription (struct RequestStruct*);
DirEndOutput (struct RequestStruct*);
DirFiles (struct FAB*);
boolean DirFileExists (struct RequestStruct*, char*, char*);
DirFilePart (struct RequestStruct*);
DirFormat (struct RequestStruct*, void*, char*, boolean);
DirFormatAcpInfoAst (struct RequestStruct*);
DirFormatLayout (struct RequestStruct*);
DirHeading (struct RequestStruct*);
int DirHttpHeader (struct RequestStruct*);
DirParseAst (struct FAB*);
DirReadMe (struct RequestStruct*,
           void(*NextTaskFunction)(struct RequestStruct*));
DirReadMeTop (struct RequestStruct*);
DirReadMeBottom (struct RequestStruct*);
DirSearchDirectories (struct RequestStruct*);
DirSearchFiles (struct RequestStruct*);

#endif /* DIR_H_LOADED */

/*****************************************************************************/
