/*****************************************************************************/
/*
                                CgiLib.h

*/
/*****************************************************************************/

#ifndef CGILIB_H_LOADED
#define CGILIB_H_LOADED 1

/* Alpha NO MEMBER ALIGNMENT unless otherwise specified */
#ifdef __ALPHA
#   pragma member_alignment __save
#   pragma nomember_alignment
#endif

#ifdef __ALPHA
#   define CGILIB_SOFTWAREID "CGILIB AXP-1.0.0";
#else
#   define CGILIB_SOFTWAREID "CGILIB VAX-1.0.0";
#endif

#ifndef CGILIB_ALL_SUPPORT
#   define CGILIB_ALL_SUPPORT 0
#endif
#if CGILIB_ALL_SUPPORT
#   define CGILIB_CGIVAR_SUPPORT 1
#   define CGILIB_CGIPLUS_SUPPORT 1
#   define CGILIB_OSU_SUPPORT 1
#   define CGILIB_POST_SUPPORT 1
#   define CGILIB_GENERAL_SUPPORT 1
#endif

/* includes code supporting CGI variables */
#ifndef CGILIB_CGIVAR_SUPPORT
#   define CGILIB_CGIVAR_SUPPORT 0
#endif

/* includes code supporting CGIplus variables */
#ifndef CGILIB_CGIPLUS_SUPPORT
#   define CGILIB_CGIPLUS_SUPPORT 0
#endif

/* includes code supporting OSU variables and output stream */
#ifndef CGILIB_OSU_SUPPORT
#   define CGILIB_OSU_SUPPORT 0
#endif

/* if false excludes code providing request body processing */
#ifndef CGILIB_POST_SUPPORT
#   define CGILIB_POST_SUPPORT 0
#endif

/* if false excludes code providing support/conversion routines */
#ifndef CGILIB_GENERAL_SUPPORT
#   define CGILIB_GENERAL_SUPPORT 0
#endif

#if CGILIB_CGIVAR_SUPPORT
    /* includes code supporting non-"WWW_" prefixed variables */
#   ifndef CGILIB_CGIVAR_NON_WWW
#      define CGILIB_CGIVAR_NON_WWW 1
#   endif
    /* if false variables will never be prefixed by "WWW_" */
#   ifndef CGILIB_CGIVAR_WWW
#      define CGILIB_CGIVAR_WWW 1
#   endif
    /* size of memory allocation in CgiLib__VarList() */
#   ifndef CGILIB_VARLIST_CHUNK
#      define CGILIB_VARLIST_CHUNK 2048
#   endif
#endif

/* if providing CGIplus variables */
#ifdef CGILIB_CGIPLUS_SUPPORT
#   ifndef CGILIB_CGIPLUS_LINE_SIZE
#      define CGILIB_CGIPLUS_LINE_SIZE 1024
#   endif
#endif

/* if providing body processing support functions */
#ifdef CGILIB_POST_SUPPORT
#   ifndef CGILIB_BODY_CHUNK
#      define CGILIB_BODY_CHUNK 4096
#   endif
    /* size of memory allocation in CgiLib__VarList() */
#   ifndef CGILIB_VARLIST_CHUNK
#      define CGILIB_VARLIST_CHUNK 2048
#   endif
#endif

/* if providing support functions */
#ifdef CGILIB_GENERAL_SUPPORT
#   ifndef CGILIB_ENCODED_CHUNK
#      define CGILIB_ENCODED_CHUNK 256
#   endif
#   ifndef CGILIB_ESCAPED_CHUNK
#      define CGILIB_ESCAPED_CHUNK 256
#   endif
#endif

#ifndef CGILIB_DEBUG
#  define CGILIB_DEBUG 0
#endif

#ifndef CGILIB_NONE
#   define CGILIB_NONE ""
#endif

/**************/
/* prototypes */
/**************/

#define CgiLibVar(vname) CgiLib__GetVar(vname,CGILIB_NONE)
#define CgiLibVarExists(vname) CgiLib__GetVar(vname,NULL)

/* for backward-compatibility */
#define CgiVar(vname) CgiLib__GetVar(vname,CGILIB_NONE)

char* CgiLibHttpStatusCodeText (int);
int CgiLibUrlDecode (char*);

#if CGILIB_POST_SUPPORT
   char* CgiLibReadRequestBody (char**, int*);
   int CgiLibFormRequestBody (char*, int);
#endif

#if CGILIB_OSU_SUPPORT
   void CgiLibOsuInit (int, char*[]);
   void CgiLibOsuStdoutCgi ();
   void CgiLibOsuStdoutRaw ();
   void CgiLibOsuStdoutText ();
#endif

#if CGILIB_GENERAL_SUPPORT
   int CgiLibHtmlEscape (char*, int, char*, int);
   int CgiLibUrlEncode (char*, int, char*, int);
#endif

/*****************************************************/
/* these functions are for INTERNAL CGILIB USE only! */
/*****************************************************/

char* CgiLib__GetVar (char*, char*);
char* CgiLib__GetKey (char*, char*);
char* CgiLib__GetForm (char*, char*);
char* CgiLib__GetPathTranslated (char*);
char* CgiLib__VarList (char*, char*, char*);

#if CGILIB_OSU_SUPPORT
   void CgiLib__OsuExitCgi ();
   void CgiLib__OsuExitRaw ();
   void CgiLib__OsuExitText ();
   char* CgiLib__OsuQueryNetLink (char*, int);
#endif

#ifdef __ALPHA
#   pragma member_alignment __restore
#endif

#endif /* CGILIB_H_LOADED */

/*****************************************************************************/

